/* $Id: //depot/branches/rmanprod/rat-6.0/ratdocs/education/RenderMan_Companion/tutorial/ch16/shaders/round.sl#1 $  (Pixar - RenderMan Division)  $Date: 2004/01/13 $ */
/*
** Copyright (c) 1998 PIXAR.  All rights reserved.  This program or
** documentation contains proprietary confidential information and trade
** secrets of PIXAR.  Reverse engineering of object code is prohibited.
** Use of copyright notice is precautionary and does not imply publication.
**
**                      RESTRICTED RIGHTS NOTICE
**
** Use, duplication, or disclosure by the Government is subject to the
** following restrictions:  For civilian agencies, subparagraphs (a) through
** (d) of the Commercial Computer Software--Restricted Rights clause at
** 52.227-19 of the FAR; and, for units of the Department of Defense, DoD
** Supplement to the FAR, clause 52.227-7013 (c)(1)(ii), Rights in
** Technical Data and Computer Software.
**
** Pixar
** 1001 West Cutting Blvd.
** Richmond, CA  94804
*/
/* Listing 16.23  Displacement shader for bevelling perpendicular
 *bilinear patches
 */
/* 
 * round(): displace the edge of a bilinear patch so that, if it is placed next
 *  to another patch at a right angle, the edge will be rounded.
 */

displacement
round(	
	float	radius = .10 )
{
	float	 uu,	/* distance in u to the nearest "vertical" edge */
		 vv,	/* distance in v to the nearest "horizontal" edge */
		 lu,	/* "real" distance to the nearest "vertical" edge */
		 lv;	/* "real" distance to the nearest "horizontal" edge */
	point	 center;/* point toward which the surface is displaced 	*/
	vector	 dpdu,	/* dPdu pointed toward patch center line 	*/
		 dpdv;	/* dPdv pointed toward patch center line 	*/

	/* Find the distance in parameter space from the nearest edge in
	   u and in v, and the directions away from those edges. */
	if (u < .5) {
		uu = u;
		dpdu = dPdu;
	} else {
		uu = 1 - u;
		dpdu = -dPdu;
	}
	if (v < .5) {
		vv = v;
		dpdv = dPdv;
	} else {
		vv = 1 - v;
		dpdv = -dPdv;
	}

	/* Find the distances from the edges in the current space. */
	lu = length(dPdu*uu);
	lv = length(dPdv*vv);

	if (lu < radius || lv < radius) { 					/* only if within radius of an edge... 						*/
	/*
	 * Find the point towards which the surface  point will be 
	 *  moved. This center is on the center line of a cylinder, if we 
	 *  are not near the corner of the patch, or is the center of a 
	 *  sphere, if we are. We move `center' to the nearest inflection 
	 *  edge along u and/or v.
	 */
		center = point(0,0,0);
		if (lu < radius)
			center += (radius-lu)*normalize(dpdu);
		if (lv < radius)
			center += (radius-lv)*normalize(dpdv);
		/* Move center perpendicular to the surface */
		center += vector(P - radius*normalize(N));
		/* Make P be distance 'radius' along the line 
                 * from 'center' to P */
		P = center + radius*normalize( P-center );
	}
	N = calculatenormal(P);
}
