#include "hyperManSL.h"
#include "hyperManFuncs.h"
#include "hyperMan_Outputs.h"

surface hyperLambert
(
string ColorMap = "";
	float UseColorMap = 0;
	float ColorMapBlur = 1;
	float ColorMapPlacement = 0;

string TransparencyMap = "";
	float UseTransparencyMap = 0;
	float TransparencyMapBlur = 1;
	color DefaultTransparency = 0;
	float TransparencyMapPlacement = 0;
	
string AmbientColorMap = "";
	float UseAmbientColorMap = 0;
	float AmbientColorMapBlur = 1;
	color DefaultAmbientColor = 0;
	float AmbientColorMapPlacement = 0;

string IncandescenceMap = "";
	float UseIncandescenceMap = 0;
	float IncandescenceMapBlur = 1;
	color DefaultIncandescence = 0;
	float IncandescenceMapPlacement = 0;	
	
string BumpMap = "";
	float UseBumpMap = 0;	
	float BumpMapBlur = 1;
	float DefaultBump = 1;
    float DoDisplacement = 0.0;
	float BumpMapPlacement = 0;	
	

	
string DiffuseMap = "";
	float UseDiffuseMap = 0;
	float DiffuseMapBlur = 1;
	float DefaultDiffuse = 0.8;
	float DiffuseMapPlacement = 0; 
	
string TranslucenceMap = "";
	float UseTranslucenceMap = 0;
	float TranslucenceMapBlur = 1;
	float DefaultTranslucence = 0;
	float TranslucenceMapPlacement = 0; 
	
string TranslucenceDepthMap = "";
	float UseTranslucenceDepthMap = 0;
	float TranslucenceDepthMapBlur = 1;
	float DefaultTranslucenceDepth = 0.5;
	float TranslucenceDepthMapPlacement = 0; 
	
string TranslucenceFocusMap = "";
	float UseTranslucenceFocusMap = 0;
	float TranslucenceFocusMapBlur = 1;
	float DefaultTranslucenceFocus = 0.5;
	float TranslucenceFocusMapPlacement = 0; 
	
string GlowintensityMap = "";
	float UseGlowintensityMap = 0;
	float GlowintensityMapBlur = 1;
	float DefaultGlowintensity = 0;
	float GlowintensityMapPlacement = 0; 
	
string MatteOpacityMode = "";
string MatteOpacityMap = "";
	float UseMatteOpacityMap = 0;
	float MatteOpacityMapBlur = 1;
	float DefaultMatteOpacity = 1;
	float MatteOpacityMapPlacement = 0; 
	
string occfilename = "";
	float occsamples = 0;
	float raymaxdist = 10;
	float cachemode = 0;
	float occflag = 0;
	
string irradfilename = "";
	float irradsamples = 0;
	float irradflag = 0;
	
string ibifilename = "";
	string ibiImage = "";
	float ibisamples = 0;
	float ibiflag = 0;

float cacheAnim = 0;

ALLOUT;
)
{
	normal Nf = faceforward( normalize(N), I );
	vector V = 0, D = 0;
	point PP = 0;
	color surfC = 0, surfT = DefaultTransparency, incC = DefaultIncandescence, Ia = 0, Id = 0, Itr = 0;
	color ambC = DefaultAmbientColor;
    float roughness = 0, diffC = DefaultDiffuse, transC = DefaultTranslucence, transDC = DefaultTranslucenceDepth; 
    float transFC = DefaultTranslucenceFocus, glowiC = DefaultGlowintensity, mattoC = DefaultMatteOpacity;
    float bmp = DefaultBump;
    color raytrace;
	
	/*if(cacheAnim != 1){*/

		point STMatrix0 = point "shader" (0,-1,1);
		point STMatrix1 = point "shader" (1,0,0);

		float ss = vector(s, t, 1) . vector(transform("shader", STMatrix0));
		float tt = vector(s, t, 1) . vector(transform("shader", STMatrix1));

		vector Rcurrent = reflect(I, Nf);
		vector Rworld = vtransform("world", Rcurrent);
		normal newNf = 0;
		newNf = doDisplace(BumpMap, UseBumpMap, BumpMapPlacement, BumpMapBlur, DefaultBump, DoDisplacement, ss, tt, Rworld, newNf);
	/*Common Material Attributes */
		surfC = doColor(ColorMap, UseColorMap, ColorMapPlacement, ColorMapBlur, ss, tt, Rworld, surfC);
		surfT = doTransparency(TransparencyMap, UseTransparencyMap, TransparencyMapPlacement, TransparencyMapBlur, ss, tt, Rworld, surfT);
		ambC = doAmbient(AmbientColorMap, UseAmbientColorMap, AmbientColorMapPlacement, AmbientColorMapBlur, ss, tt, Rworld, DefaultAmbientColor, ambC);
		incC = doIncandescence(IncandescenceMap, UseIncandescenceMap, IncandescenceMapPlacement, IncandescenceMapBlur, ss, tt, Rworld, DefaultIncandescence, incC);
		diffC = doDiffuse(DiffuseMap, UseDiffuseMap, DiffuseMapPlacement,	DiffuseMapBlur, DefaultDiffuse, ss, tt, Rworld, diffC);
		transC = doTranslucence(TranslucenceMap, UseTranslucenceMap, TranslucenceMapPlacement,	TranslucenceMapBlur, DefaultTranslucence, ss, tt, Rworld, transC);
		transDC = doTranslucenceDepth(TranslucenceDepthMap, UseTranslucenceDepthMap, TranslucenceDepthMapPlacement,	TranslucenceDepthMapBlur, DefaultTranslucenceDepth, ss, tt, Rworld, transDC);
		transFC = doTranslucenceFocus(TranslucenceFocusMap, UseTranslucenceFocusMap, TranslucenceFocusMapPlacement,	TranslucenceFocusMapBlur, DefaultTranslucenceFocus, ss, tt, Rworld, transFC);

	/*Special Effects*/	
		glowiC = doGlowintensity(GlowintensityMap, UseGlowintensityMap, GlowintensityMapPlacement,	GlowintensityMapBlur, DefaultGlowintensity, ss, tt, Rworld, glowiC);

	/*Matte Opacity*/	
		mattoC = doMatteOpacity(MatteOpacityMap, UseMatteOpacityMap, MatteOpacityMapPlacement,	MatteOpacityMapBlur, DefaultMatteOpacity, ss, tt, Rworld, mattoC);   

		Ia = hyperManSLAmbient(Nf, ambC);
		Id = diffC * diffuse(normalize(Nf)); 
		Itr = hyperManSLTranslucence(Nf,transC,transFC, transDC);

		Oamb += Cs * Ia;
		Odiff += Cs * Id;
		Oalpha = mattoC * surfT;	    

		if(occflag == 1){
		Oocc = occsurf(occfilename, occsamples, raymaxdist, cachemode);
		raytrace = Oocc;}
		if(irradflag == 1){
		Oirrad = irradsurf(irradfilename, irradsamples, raymaxdist, cachemode);
		raytrace = Oirrad;}
		if(ibiflag == 1){
		Oibi = ibisurf(ibifilename, ibiImage, ibisamples, raymaxdist, cachemode);
		raytrace = Oibi;}

				Oi = surfT;
				if((occflag == 1)||(irradflag == 1)||(ibiflag == 1)){
				Ci = surfT *  raytrace * surfC * (Ia + Id + Itr);}
				else {
				Ci = surfT * surfC * (Ia + Id + Itr);}

	
	        if(cacheAnim == 1){
			if(occflag == 1){
			Oocc = occsurf(occfilename, occsamples, raymaxdist, cachemode);}
			if(irradflag == 1){
			Oirrad = irradsurf(irradfilename, irradsamples, raymaxdist, cachemode);}
			if(ibiflag == 1){
			Oibi = ibisurf(ibifilename, ibiImage, ibisamples, raymaxdist, cachemode);}
	}
}

    