#include "hyperMan_Outputs.h"
#ifndef _H_hyperManSL
#define _H_hyperManSL

/* ---------------------------------- * ---------------------------------- *

Note hyperManSL.h is a modified version of walias.h Avinash Sunnasy

$Revision: #1 $  $Date: 2004/01/13 $
---------from Alias Rendering Gurus--------------------------
> Notation:  A.B represents the dot product of the two vectors A and B
>	    x^^y raises x to the power y
>
> ESSENTIAL PHONG SHADING
> -----------------------
>
> The following formula does not include translucence, or incandescence, but
> should give the basics of the Alias Phong Shading Model.
> It is essentially a more flop optimal version of the basic (L.R)^^c fomula.
> Only one light is taken account of in the formulas below, but obviously there
> is a loop over all affecting lights where Ia, Id and Is are summed and then
> condensed into If.
>
> Let:
>   N   be the Normal of the surface
>   L   be the unit vector pointing to the light from the surface
>   R   be the perfect specular reflection direction
>   c   be the "shinyness" value from the interface
>   d   be the "diffuse" value from the interface
>   s   be the "specular" value from the interface (RGB)
>   r   be the "reflectivity" value from the interface
>   l   be the intensity of the light at this point in space (RGB)
>   t   be the transparent color of the surface (RGB)
>   Ia  be the computed ambient intensity (RGB)
>   Id  be the computed diffuse intensity (RGB)
>   Is  be the computed specular intensity (RGB)
>   Ir  be the color in the reflected direction (RGB)
>   It  be the color in the refracted (transmission) direction (RGB)
>   If  be the computed final intensity  (RGB)
>
> Ia = ambient * surf_color
> Id = L.N * D * surf_color * l
>
> (log(L.R) * c) >= -12.5 : s * l * exp(log(L.R) * c)
>	L.R > 0   : {
> Is = {  (log(L.R) * c) < -12.5  :  0
>	L.R <= 0  :  0
>
> If = Ia + Id + Is + (Ir * s * r) + (It * (t - (Ia + Id)))
>
> ESSENTIAL BLINN SHADING
> -----------------------
>
> The following formula does not include transparency, reflection, translucence,
> or incandescence, but should give the basics of the Alias Blinn Shading Model.
> It is a kludgy version of the Torrance-Sparrow model as described by Blinn's
> SIGGRAPH '78 paper. (Transparency and reflection are handled much as they are
> in the Phong model above.)
>
> Let:
>   N   be the Normal of the surface
>   L   be the unit vector pointing to the light from the surface
>   R   be the perfect specular reflection direction
>   S   be the unit vector pointing to the eye from this point
>   H   be the bisector of S and L
>   d   be the "diffuse" value from the interface
>   s   be the "specular" value from the interface (RGB)
>   r   be the "specular rolloff" value from the interface
>   e   be the "eccentricity" value from the interface
>   l   be the intensity of the light at this point in space (RGB)
>   Ia  be the computed ambient intensity (RGB)
>   Id  be the computed diffuse intensity (RGB)
>   Is  be the computed specular intensity (RGB)
>   It  be the computed total intensity (RGB)
>   Dd  temporary storage for values
>   Gg  temporary storage for values
>   Ff  temporary storage for values
>   nh  temporary storage for values
>   nh2 temporary storage for values
>   E   temporary storage for values
>   t   temporary storage for values
>
> Ia = ambient * surf_color
> Id = L.N * D * surf_color * l
>
> E  = 1 / (e * e - 1)
> nh2 = (N.H)^^2
> Dd = [ (E + 1) / (nh2 + E) ]^^2
>
> nh = N.H * 2
> if ( S.N < L.N )
>     if ( S.N * nh < S.H )
>	 Gg = nh / (S.H)
>     else
>	 Gg = 1 / S.N
> else
>     if ( L.N * nh < S.H )
>	 Gg = ( L.N * nh ) / (S.H * S.N)
>     else
>	 Gg = 1 / S.N
>
> # What follows is a poor man's Fresnel
> t = ( 1 - S.H )^^3
> Ff = t + ( 1 - t ) * ( 1 - r )
DB: this was experimentally determined to be wrong, actually, we want:
> Ff = t + ( 1 - t ) * r
>
> if ( Dd * Gg * Ff > 1E-5 )
>     Is = s * l * Dd * Gg * Ff
> else
>     Is = 0
>
> It = Ia + Id + Is

* ---------------------------------- * ---------------------------------- */

#define HALF_PI 1.57079632679
color
hyperManSLAmbient( normal Nf;  color ambientColor; )
{
    extern point P;
    extern vector L;
    extern color Cl;
    float nondif, nonspec, ambientShade;
    vector Ln;
    color Ctot, Cpt;

    Ctot = ambient() + ambientColor;
    illuminance(P, Nf, PI)
    {
	if( 0 == lightsource("__nondiffuse", nondif) )
	    nondif = 0;
	if( 0 == lightsource("__nonspecular", nonspec) )
	    nonspec = 0;
	if( 0 == lightsource("ambientShade", ambientShade) )
	    ambientShade = 0;
	if( nondif == 1 && nonspec == 1 && ambientShade != 0 )
	{
	    Ln = normalize(L);
	    Cpt = Cl * mix(1, Nf.Ln, ambientShade);
	    Ctot += Cpt;
	}
    }
    return Ctot;
}

/* Func adjusted to deal with Maya trans focus & depth  Avinash Sunnasy (c)*/

color
hyperManSLTranslucence( normal Nf; float c, d , depth; )
{
    color C = 0;
    extern point P;
    extern color Cl;
    float nondiff;
	float angle = (PI*2)*d;
	vector newDepth = L * depth;
	point newPoint = P + newDepth;
    if( c != 0 )
    {
	illuminance(newPoint, Nf, angle)
	{
	    if( 0 == lightsource("__nondiffuse", nondiff) )
		nondiff = 0;
	    C += (1 - nondiff) * Cl;
	}
	C *= c;
    }
    return C;
}

/* std phong: C += Cl * pow(max(0.0,R.Ln), size); */
color
hyperManSLPhong(vector R; normal Nf; float c;
	    output color Cr; )
{
    color C = 0;
    vector Ln;
    float nonspec, nondiff, k;
    extern vector L;
    extern point P;
    extern color Cl;

    Cr = 0;
    illuminance(P, Nf, HALF_PI)
    {
	if( 0 == lightsource("__nonspecular", nonspec) )
	    nonspec = 0;
	if(nonspec < 1 )
	{
	    Ln = normalize(L);
	    k = log(Ln.R) * c;
	    if(k >= -12.5)
	    {
		C += Cl * (1 - nonspec) * exp(log(Ln.R) * c);
	    }
	    /* now look for environment reflections.  These
		are indicated by lights which are specular AND
		nondiffuse */
	    if( 0 != lightsource("__nondiffuse", nondiff) )
	    {
		Cr += (1 - nonspec) * nondiff * Cl;
	    }
	}
    }
    return C;
}

color
hyperManSLBlinn( normal Nf;
	     float eccentricity, specularRollOff;
	    output color Cr; )
{
     float E = 1 / (eccentricity * eccentricity - 1);
    color C = 0;
    vector H, Ln, V, Nn;
    float NH, NH2, NHSQ, Dd, Gg, VN, VH, LN, Ff, tmp;
    float nondiff, nonspec;
    extern point P;
    extern vector L, I;
    extern color Cl;

    Cr = 0;
    V = normalize(-I);
    VN = V.Nf;
    illuminance(P, Nf, HALF_PI)
    {
	if( 0 == lightsource("__nonspecular", nonspec) )
	    nonspec = 0;
	if( nonspec < 1 )
	{
	    Ln = normalize(L);
	    H = normalize(Ln+V);
	    NH = Nf.H;
	    NHSQ = NH*NH;
	    NH2 = NH * 2;
	    Dd = (E+1) / (NHSQ + E);
	    Dd *= Dd;
	    VH = V.H;
	    LN = Ln.Nf;
	    if( VN < LN )
	    {
		if( VN * NH2 < VH )
		    Gg = NH2 / VH;
		else
		    Gg = 1 / VN;
	    }
	    else
	    {
		if( LN * NH2 < VH )
		    Gg = (LN * NH2) / (VH * VN);
		else
		    Gg = 1 / VN;
	    }
	    /* poor man's Fresnel */
	    tmp = pow((1 - VH), 3);
	    Ff = tmp + (1 - tmp) * specularRollOff;
	    C += Cl * Dd * Gg * Ff;

	    /* now look for environment reflections.  These
		are indicated by lights which are specular AND
		nondiffuse */
	    if( 0 != lightsource("__nondiffuse", nondiff) )
	    {
		Cr += (1 - nonspec) * nondiff * Ff * Cl;
	    }
	}
    }
    return C;
}

#define WALIAS_GLOWPARMS \
    float    glowStyle = 0; \
    color    glowColor = lightcolor; \
    float    glowIntensity = 1; \
    float    glowSpread = .5; \
    float    glowStarLevel = 0; \
    float    glowRadialNoise = 0; \
    float    glowNoiseFreq = 0; \
    float    glowRotation = 0; \
    float    glowStarPoints = 4


#endif
